# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_list(runner, product_id):
    runner.invoke(["topic-create", "--name", "osp", "--product-id", product_id])
    runner.invoke(["topic-create", "--name", "ovirt", "--product-id", product_id])
    topics = runner.invoke(["topic-list"])["topics"]

    assert len(topics) == 2
    # assert topics[0]['name'] == 'ovirt'
    # assert topics[1]['name'] == 'osp'


def test_create(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["name"] == "osp"
    assert len(topic["component_types"]) == 0


def test_create_with_component_types(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--component_types",
            "foo,bar",
        ]
    )["topic"]
    assert topic["name"] == "osp"
    assert len(topic["component_types"]) == 2
    assert topic["component_types"][0] == "foo"
    assert topic["component_types"][1] == "bar"


def test_create_with_data(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--data",
            '{"foo": "bar"}',
        ]
    )["topic"]
    assert topic["name"] == "osp"
    assert topic["data"]["foo"] == "bar"


def test_create_inactive(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id, "--no-active"]
    )["topic"]
    assert topic["state"] == "inactive"


def test_create_export_control_default(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["export_control"] is False


def test_create_export_control(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--export-control",
        ]
    )["topic"]
    assert topic["export_control"] is True


def test_create_no_export_control(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--no-export-control",
        ]
    )["topic"]
    assert topic["export_control"] is False


def test_delete(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    result = runner.invoke_raw(["topic-delete", topic["id"]])

    assert result.status_code == 204


def test_show(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    topic = runner.invoke(["topic-show", topic["id"]])["topic"]

    assert topic["name"] == "osp"


def test_attach_team(runner, product_id):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    assert team["name"] == "foo"

    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["name"] == "osp"

    topic_team = runner.invoke(
        ["topic-attach-team", topic["id"], "--team-id", team["id"]]
    )
    assert topic_team["team_id"] == team["id"]
    assert topic_team["topic_id"] == topic["id"]


def test_list_team(runner, product_id):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    assert team["name"] == "foo"

    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["name"] == "osp"

    topic_team = runner.invoke(
        ["topic-attach-team", topic["id"], "--team-id", team["id"]]
    )
    assert topic_team["team_id"] == team["id"]
    assert topic_team["topic_id"] == topic["id"]

    result = runner.invoke(["topic-list-team", topic["id"]])["teams"]
    assert len(result) == 1
    assert result[0]["name"] == "foo"


def test_unattach_team(runner, product_id):
    team = runner.invoke(["team-create", "--name", "foo"])["team"]
    assert team["name"] == "foo"

    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["name"] == "osp"

    topic_team = runner.invoke(
        ["topic-attach-team", topic["id"], "--team-id", team["id"]]
    )
    assert topic_team["team_id"] == team["id"]
    assert topic_team["topic_id"] == topic["id"]

    result = runner.invoke(["topic-list-team", topic["id"]])["teams"]
    assert len(result) == 1
    assert result[0]["name"] == "foo"

    topic_team = runner.invoke_raw(
        ["topic-unattach-team", topic["id"], "--team-id", team["id"]]
    )

    teams = runner.invoke(["topic-list-team", topic["id"]])["teams"]
    assert len(teams) == 0


def test_update_active(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    assert topic["state"] == "active"

    result = runner.invoke(
        ["topic-update", topic["id"], "--etag", topic["etag"], "--no-active"]
    )

    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["state"] == "inactive"

    result = runner.invoke(
        [
            "topic-update",
            topic["id"],
            "--etag",
            result["topic"]["etag"],
            "--name",
            "foobar",
        ]
    )

    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["state"] == "inactive"
    assert result["topic"]["name"] == "foobar"

    result = runner.invoke(
        ["topic-update", topic["id"], "--etag", result["topic"]["etag"], "--active"]
    )

    assert result["topic"]["state"] == "active"


def test_update_with_component_types(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    result = runner.invoke(
        [
            "topic-update",
            topic["id"],
            "--etag",
            topic["etag"],
            "--component_types",
            "foo,bar",
        ]
    )

    assert result["topic"]["id"] == topic["id"]
    assert len(result["topic"]["component_types"]) == 2
    assert result["topic"]["component_types"][0] == "foo"
    assert result["topic"]["component_types"][1] == "bar"


def test_update_with_data(runner, product_id):
    topic = runner.invoke(
        ["topic-create", "--name", "osp", "--product-id", product_id]
    )["topic"]
    result = runner.invoke(
        [
            "topic-update",
            topic["id"],
            "--etag",
            topic["etag"],
            "--data",
            '{"foo": "bar"}',
        ]
    )

    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["data"]["foo"] == "bar"


def test_update_export_control_no_change(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--export-control",
        ]
    )["topic"]
    result = runner.invoke(["topic-update", topic["id"], "--etag", topic["etag"]])
    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["export_control"] == topic["export_control"]


def test_update_no_export_control_no_change(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--no-export-control",
        ]
    )["topic"]
    result = runner.invoke(["topic-update", topic["id"], "--etag", topic["etag"]])
    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["export_control"] == topic["export_control"]


def test_update_export_control(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--export-control",
        ]
    )["topic"]
    result = runner.invoke(
        ["topic-update", topic["id"], "--etag", topic["etag"], "--no-export-control"]
    )
    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["export_control"] is False


def test_update_no_export_control(runner, product_id):
    topic = runner.invoke(
        [
            "topic-create",
            "--name",
            "osp",
            "--product-id",
            product_id,
            "--no-export-control",
        ]
    )["topic"]
    result = runner.invoke(
        ["topic-update", topic["id"], "--etag", topic["etag"], "--export-control"]
    )
    assert result["topic"]["id"] == topic["id"]
    assert result["topic"]["export_control"] is True


def test_where_on_list(runner, product_id):
    runner.invoke(["topic-create", "--name", "osp1", "--product-id", product_id])
    runner.invoke(["topic-create", "--name", "osp2", "--product-id", product_id])
    assert runner.invoke(["topic-list", "--where", "name:osp1"])["_meta"]["count"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import codecs
import os
import setuptools

from dciclient import version


def _get_requirements():
    requirements_path = "%s/%s" % (
        os.path.dirname(os.path.abspath(__file__)),
        "requirements.txt",
    )
    with open(requirements_path, "r") as f:
        requirements = f.read()
        # remove the dependencies which comes from url source because
        # it's not supported by install_requires
        return [dep for dep in requirements.split("\n") if not dep.startswith("-e")]


def _get_readme():
    readme_path = "%s/%s" % (os.path.dirname(os.path.abspath(__file__)), "README.md")

    with codecs.open(readme_path, "r", encoding="utf8") as f:
        return f.read()


setuptools.setup(
    name="dciclient",
    version=version.__version__,
    packages=setuptools.find_packages(),
    author="Distributed CI team",
    author_email="distributed-ci@redhat.com",
    description="Python client for DCI Control Server",
    long_description=_get_readme(),
    long_description_content_type="text/markdown",
    install_requires=_get_requirements(),
    url="https://github.com/redhat-cip/dci-control-server",
    license="Apache v2.0",
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Topic :: System :: Distributed Computing",
    ],
    entry_points={"console_scripts": ["dcictl = dciclient.shell:main"]},
)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1.api.tag import add_tag_to_resource, delete_tag_from_resource

RESOURCE = "components"


def create(
    context,
    name,
    type,
    topic_id,
    canonical_project_name=None,
    data={},
    title=None,
    message=None,
    url=None,
    state="active",
):
    return base.create(
        context,
        RESOURCE,
        name=name,
        type=type,
        canonical_project_name=canonical_project_name,
        data=data,
        title=title,
        message=message,
        url=url,
        topic_id=topic_id,
        state=state,
    )


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(context, id=None, etag=None, name=None, content=None, state=None, data=None):
    return base.update(
        context,
        RESOURCE,
        id=id,
        etag=etag,
        name=name,
        content=content,
        state=state,
        data=data,
    )


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def file_upload(context, id, file_path):
    uri = "%s/%s/%s/files" % (context.dci_cs_api, RESOURCE, id)
    with open(file_path, "rb") as f:
        return context.session.post(uri, data=f)


def file_get(context, id, file_id):
    uri = "%s/%s/%s/files/%s" % (context.dci_cs_api, RESOURCE, id, file_id)
    return context.session.get(uri)


def file_download(context, id, file_id, target):
    uri = "%s/%s/%s/files/%s/content" % (context.dci_cs_api, RESOURCE, id, file_id)
    return base.download(context, uri, target)


def file_list(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="files", **kwargs)


def file_delete(context, id, file_id):
    return base.delete(
        context, RESOURCE, id, subresource="files", subresource_id=file_id
    )


def list_issues(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource="issues", **kwargs)


def attach_issue(context, id, url):
    uri = "%s/%s/%s/issues" % (context.dci_cs_api, RESOURCE, id)
    data = {"url": url}
    return context.session.post(uri, json=data)


def unattach_issue(context, id, issue_id):
    return base.delete(
        context, RESOURCE, id, subresource="issues", subresource_id=issue_id
    )


def add_tag(context, id, name):
    return add_tag_to_resource(context, RESOURCE, id, name)


def delete_tag(context, id, tag_id):
    return delete_tag_from_resource(context, RESOURCE, id, tag_id)



# -*- encoding: utf-8 -*-
#
# Copyright Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from argparse import ArgumentParser
from dciclient.version import __version__

_default_dci_cs_url = "http://127.0.0.1:5000"
_default_sso_url = "http://127.0.0.1:8180"


def _create_boolean_flags(parser, flags, default, dest=None):
    flags = flags.split("/")
    dest = dest if dest else flags[0].strip("--")
    group = parser.add_mutually_exclusive_group()
    group.add_argument(flags[0], action="store_true", default=default, dest=dest)
    group.add_argument(flags[1], action="store_false", dest=dest)


def parse_arguments(args, environment={}):
    base_parser = ArgumentParser(add_help=False)
    base_parser.add_argument("--verbose", "--long", default=False, action="store_true")

    parser = ArgumentParser(prog="dcictl")
    parser.add_argument(
        "--version", action="version", version="%(prog)s " + __version__
    )
    parser.add_argument(
        "--dci-login",
        default=environment.get("DCI_LOGIN", None),
        help="DCI login or 'DCI_LOGIN' environment variable.",
    )
    parser.add_argument(
        "--dci-password",
        default=environment.get("DCI_PASSWORD", None),
        help="DCI password or 'DCI_PASSWORD' environment variable.",
    )
    parser.add_argument(
        "--dci-client-id",
        default=environment.get("DCI_CLIENT_ID", None),
        help="DCI CLIENt ID or 'DCI_CLIENT_ID' environment variable.",
    )
    parser.add_argument(
        "--dci-api-secret",
        default=environment.get("DCI_API_SECRET", None),
        help="DCI API secret or 'DCI_API_SECRET' environment variable.",
    )
    parser.add_argument(
        "--dci-cs-url",
        default=environment.get("DCI_CS_URL", _default_dci_cs_url),
        help="DCI control server url, default to '%s'." % _default_dci_cs_url,
    )
    parser.add_argument(
        "--sso-url",
        default=environment.get("SSO_URL", _default_sso_url),
        help="SSO url, default to '%s'." % _default_sso_url,
    )
    parser.add_argument(
        "--sso-username",
        default=environment.get("SSO_USERNAME"),
        help="SSO username or 'SSO_USERNAME' environment variable.",
    )
    parser.add_argument(
        "--sso-password",
        default=environment.get("SSO_PASSWORD"),
        help="SSO password or 'SSO_PASSWORD' environment variable.",
    )
    parser.add_argument(
        "--sso-token",
        default=environment.get("SSO_TOKEN"),
        help="SSO token or 'SSO_TOKEN' environment variable.",
    )
    parser.add_argument(
        "--refresh-sso-token",
        default=False,
        action="store_true",
        help="Refresh the token",
    )
    parser.add_argument(
        "--format",
        default="table",
        choices=["table", "json", "csv", "tsv"],
        help="Output format",
    )

    subparsers = parser.add_subparsers()
    # user commands
    p = subparsers.add_parser(
        "user-list", help="List all users.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="user-list")

    p = subparsers.add_parser(
        "user-create", help="Create a user.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--password", required=True)
    p.add_argument("--email", required=True)
    p.add_argument("--fullname")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.add_argument("--team-id")
    p.set_defaults(command="user-create")

    p = subparsers.add_parser(
        "user-update", help="Update a user.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--fullname", default="")
    p.add_argument("--email")
    p.add_argument("--password")
    p.add_argument("--team-id")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="user-update")

    p = subparsers.add_parser(
        "user-delete", help="Update a user.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="user-delete")

    p = subparsers.add_parser("user-show", help="Show a user.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="user-show")

    # team commands
    p = subparsers.add_parser(
        "team-list", help="List all teams.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="team-list")

    p = subparsers.add_parser(
        "team-create", help="Create a team.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--country")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.set_defaults(command="team-create")

    p = subparsers.add_parser(
        "team-update", help="Update a team.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--country")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    _create_boolean_flags(p, "--external/--no-external", default=None, dest="external")
    p.set_defaults(command="team-update")

    p = subparsers.add_parser(
        "team-delete", help="Update a team.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="team-delete")

    p = subparsers.add_parser("team-show", help="Show a team.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="team-show")

    # analytic commands
    p = subparsers.add_parser(
        "analytic-list", help="List all analytics.", parents=[base_parser]
    )
    p.add_argument("--job-id")
    p.set_defaults(command="analytic-list")

    p = subparsers.add_parser(
        "analytic-create", help="Create a analytic.", parents=[base_parser]
    )
    p.add_argument("--job-id", required=True)
    p.add_argument("--name", required=True)
    p.add_argument("--type", required=True)
    p.add_argument("--url")
    p.add_argument("--data", default="{}")
    p.set_defaults(command="analytic-create")

    p = subparsers.add_parser(
        "analytic-update", help="Update a analytic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--job-id", required=True)
    p.add_argument("--name", required=True)
    p.add_argument("--type", required=True)
    p.add_argument("--url")
    p.add_argument("--data", default="{}")
    p.set_defaults(command="analytic-update")

    p = subparsers.add_parser(
        "analytic-show", help="Show a analytic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--job-id", required=True)
    p.set_defaults(command="analytic-show")

    # product commands
    p = subparsers.add_parser(
        "product-list", help="List all products.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="product-list")

    p = subparsers.add_parser(
        "product-create", help="Create a product.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--label")
    p.add_argument("--description")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.set_defaults(command="product-create")

    p = subparsers.add_parser(
        "product-update", help="Update a product.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--label")
    p.add_argument("--description")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    _create_boolean_flags(p, "--external/--no-external", default=None, dest="external")
    p.set_defaults(command="product-update")

    p = subparsers.add_parser(
        "product-delete", help="Update a product.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="product-delete")

    p = subparsers.add_parser(
        "product-show", help="Show a product.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="product-show")

    p = subparsers.add_parser(
        "product-attach-team", help="Attach team to a product.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--team-id")
    p.set_defaults(command="product-attach-team")

    p = subparsers.add_parser(
        "product-detach-team", help="Detach team to a product.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--team-id")
    p.set_defaults(command="product-detach-team")

    p = subparsers.add_parser(
        "product-list-teams",
        help="List all teams attached to a product.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="product-list-teams")

    # feeder commands
    p = subparsers.add_parser(
        "feeder-list", help="List all feeders.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="feeder-list")

    p = subparsers.add_parser(
        "feeder-create", help="Create a feeder.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--data")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.add_argument("--team-id")
    p.set_defaults(command="feeder-create")

    p = subparsers.add_parser(
        "feeder-update", help="Update a feeder.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--data")
    p.add_argument("--team-id")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="feeder-update")

    p = subparsers.add_parser(
        "feeder-delete", help="Update a feeder.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="feeder-delete")

    p = subparsers.add_parser(
        "feeder-show", help="Show a feeder.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="feeder-show")

    p = subparsers.add_parser(
        "feeder-reset-api-secret",
        help="reset api secret for a feeder.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="feeder-reset-api-secret")

    # topic commands
    p = subparsers.add_parser(
        "topic-list", help="List all topics.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="topic-list")

    p = subparsers.add_parser(
        "topic-create", help="Create a topic.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--product-id")
    p.add_argument(
        "--component_types", default=None, help="Component types separated by commas."
    )
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    _create_boolean_flags(
        p, "--export-control/--no-export-control", default=False, dest="export_control"
    )
    p.add_argument("--data")
    p.set_defaults(command="topic-create")

    p = subparsers.add_parser(
        "topic-update", help="Update a topic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument(
        "--component_types", default=None, help="Component types separated by commas."
    )
    p.add_argument("--label")
    p.add_argument("--next-topic-id")
    _create_boolean_flags(p, "--active/--no-active", default=False, dest="state")
    _create_boolean_flags(
        p, "--export-control/--no-export-control", default=None, dest="export_control"
    )
    p.add_argument("--product-id")
    p.add_argument("--data")
    p.set_defaults(command="topic-update")

    p = subparsers.add_parser(
        "topic-delete", help="Delete a topic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="topic-delete")

    p = subparsers.add_parser("topic-show", help="Show a topic.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="topic-show")

    p = subparsers.add_parser(
        "topic-attach-team", help="Attach a team to a topic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--team-id")
    p.set_defaults(command="topic-attach-team")

    p = subparsers.add_parser(
        "topic-unattach-team", help="Unattach a team to a topic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--team-id")
    p.set_defaults(command="topic-unattach-team")

    p = subparsers.add_parser(
        "topic-list-team", help="List teams attached to a topic.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="topic-list-team")

    # jobstate commands
    p = subparsers.add_parser(
        "jobstate-show", help="Show a jobstate.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="jobstate-show")

    # component commands
    p = subparsers.add_parser(
        "component-list", help="List all components.", parents=[base_parser]
    )
    p.add_argument("--topic-id", required=True, dest="id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="component-list")

    p = subparsers.add_parser(
        "component-create", help="Create a component.", parents=[base_parser]
    )
    p.add_argument("--name", required=True, help="Name of component")
    p.add_argument("--type", required=True, help="Type of component")
    p.add_argument(
        "--canonical_project_name", default=None, help="Canoncial project name."
    )
    p.add_argument("--title", help="Title of component")
    p.add_argument("--message", help="Component message")
    p.add_argument("--url", help="URL to look for the component")
    p.add_argument("--topic-id", required=True, help="Topic ID")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.add_argument("--data", default="{}", help="Data to pass (JSON)")
    p.set_defaults(command="component-create")

    p = subparsers.add_parser(
        "component-update", help="Update a component.", parents=[base_parser]
    )
    p.add_argument("id")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="component-update")

    p = subparsers.add_parser(
        "component-delete", help="Delete a component.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="component-delete")

    p = subparsers.add_parser(
        "component-show", help="Show a component.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="component-show")

    p = subparsers.add_parser(
        "component-attach-issue",
        help="Attach an issue to a component.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--url", required=True)
    p.set_defaults(command="component-attach-issue")

    p = subparsers.add_parser(
        "component-unattach-issue",
        help="Unattach an issue to a component.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--issue-id", required=True)
    p.set_defaults(command="component-unattach-issue")

    p = subparsers.add_parser(
        "component-list-issue",
        help="List all component attached issues.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.set_defaults(command="component-list-issue")

    p = subparsers.add_parser(
        "component-file-upload",
        help="Attach a file to a component.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--path", required=True)
    p.set_defaults(command="component-file-upload")

    p = subparsers.add_parser(
        "component-file-show", help="Show a component file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.set_defaults(command="component-file-show")

    p = subparsers.add_parser(
        "component-file-download",
        help="Retrieve a component file.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.add_argument("--target", required=True)
    p.set_defaults(command="component-file-download")

    p = subparsers.add_parser(
        "component-file-list",
        help="List files attached to a component.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="component-file-list")

    p = subparsers.add_parser(
        "component-file-delete", help="Delete a component file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.set_defaults(command="component-file-delete")

    # file commands
    p = subparsers.add_parser(
        "file-list", help="List all files.", parents=[base_parser]
    )
    p.add_argument("job_id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="file-list")

    p = subparsers.add_parser("file-show", help="Show a file.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="file-show")

    p = subparsers.add_parser(
        "file-delete", help="Delete a file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="file-delete")

    # job commands
    p = subparsers.add_parser("job-list", help="List all jobs.", parents=[base_parser])
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=10)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="job-list")

    p = subparsers.add_parser("job-show", help="Show a job.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="job-show")

    p = subparsers.add_parser("job-delete", help="Delete a job.", parents=[base_parser])
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="job-delete")

    p = subparsers.add_parser(
        "job-results", help="List all job results.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.set_defaults(command="job-results")

    p = subparsers.add_parser(
        "job-attach-issue", help="Attach an issue to a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--url", required=True)
    p.set_defaults(command="job-attach-issue")

    p = subparsers.add_parser(
        "job-unattach-issue", help="Unattach an issue to a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--issue-id", required=True)
    p.set_defaults(command="job-unattach-issue")

    p = subparsers.add_parser(
        "job-list-issue", help="List all job attached issues.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="job-list-issue")

    p = subparsers.add_parser(
        "job-output", help="Show the job output.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="job-output")

    p = subparsers.add_parser(
        "job-list-test", help="List all tests attached issues.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="job-list-issue")

    p = subparsers.add_parser(
        "job-add-tag", help="Add a tag to a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("name")
    p.set_defaults(command="job-add-tag")

    p = subparsers.add_parser(
        "job-delete-tag", help="Drop a tag to a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("tag_id")
    p.set_defaults(command="job-delete-tag")

    p = subparsers.add_parser(
        "job-list-tags", help="List all the tags of a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="job-list-tags")

    p = subparsers.add_parser(
        "job-upload-file", help="Attach a file to a job.", parents=[base_parser]
    )
    p.add_argument("job_id")
    p.add_argument("--name", required=True)
    p.add_argument("--path", required=True, dest="file_path")
    p.add_argument("--jobstate-id")
    p.add_argument("--test-id")
    p.add_argument("--mime")
    p.set_defaults(command="job-upload-file")

    p = subparsers.add_parser(
        "job-download-file", help="Retrieve a job file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.add_argument("--target", required=True)
    p.set_defaults(command="job-download-file")

    p = subparsers.add_parser(
        "job-show-file", help="Show a job file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.set_defaults(command="job-show-file")

    p = subparsers.add_parser(
        "job-list-file", help="List files attached to a job.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="job-list-file")

    p = subparsers.add_parser(
        "job-delete-file", help="Delete a job file.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--file-id", required=True)
    p.set_defaults(command="job-delete-file")

    # test commands
    p = subparsers.add_parser(
        "test-list", help="List all tests.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="test-list")

    p = subparsers.add_parser(
        "test-create", help="Create a test.", parents=[base_parser]
    )
    p.add_argument("--name")
    p.add_argument("--data", default="{}")
    _create_boolean_flags(p, "--active/--no-active", default=True, dest="state")
    p.set_defaults(command="test-create")

    p = subparsers.add_parser(
        "test-update", help="Update a test.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--data", default="{}")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="test-update")

    p = subparsers.add_parser(
        "test-delete", help="Delete a test.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="test-delete")

    p = subparsers.add_parser("test-show", help="Show a test.", parents=[base_parser])
    p.add_argument("id")
    p.set_defaults(command="test-show")

    # remoteci commands
    p = subparsers.add_parser(
        "remoteci-list", help="List all remotecis.", parents=[base_parser]
    )
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="remoteci-list")

    p = subparsers.add_parser(
        "remoteci-create", help="Create a remoteci.", parents=[base_parser]
    )
    p.add_argument("--name", required=True)
    p.add_argument("--team-id", required=False)
    p.add_argument("--data", default="{}")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="remoteci-create")

    p = subparsers.add_parser(
        "remoteci-update", help="Update a remoteci.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.add_argument("--name")
    p.add_argument("--team-id")
    p.add_argument("--data", default="{}")
    _create_boolean_flags(p, "--active/--no-active", default=None, dest="state")
    p.set_defaults(command="remoteci-update")

    p = subparsers.add_parser(
        "remoteci-delete", help="Delete a remoteci.", parents=[base_parser]
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="remoteci-delete")

    p = subparsers.add_parser(
        "remoteci-show", help="Show a remoteci.", parents=[base_parser]
    )
    p.add_argument("id")
    p.set_defaults(command="remoteci-show")

    p = subparsers.add_parser(
        "remoteci-get-data",
        help="Retrieve data field from a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--keys", default=None)
    p.set_defaults(command="remoteci-get-data")

    p = subparsers.add_parser(
        "remoteci-attach-test",
        help="Attach a test to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--test-id", required=True)
    p.set_defaults(command="remoteci-attach-test")

    p = subparsers.add_parser(
        "remoteci-unattach-test",
        help="Unattach a test to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--test-id", required=True)
    p.set_defaults(command="remoteci-unattach-test")

    p = subparsers.add_parser(
        "remoteci-list-test",
        help="List tests attached to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="remoteci-list-test")

    p = subparsers.add_parser(
        "remoteci-reset-api-secret",
        help="Reset a remoteci api secret.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="remoteci-reset-api-secret")

    p = subparsers.add_parser(
        "remoteci-refresh-keys",
        help="Refresh a remoteci key pair.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--etag", required=True)
    p.set_defaults(command="remoteci-refresh-keys")

    p = subparsers.add_parser(
        "remoteci-attach-user",
        help="Attach a user to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--user-id", required=True)
    p.set_defaults(command="remoteci-attach-user")

    p = subparsers.add_parser(
        "remoteci-unattach-user",
        help="Unattach a user to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--user-id", required=True)
    p.set_defaults(command="remoteci-unattach-user")

    p = subparsers.add_parser(
        "remoteci-list-user",
        help="List users attached to a remoteci.",
        parents=[base_parser],
    )
    p.add_argument("id")
    p.add_argument("--sort", default="-created_at")
    p.add_argument("--limit", default=50)
    p.add_argument("--offset", default=0)
    p.add_argument("--where", help="Optional filter criteria", required=False)
    p.set_defaults(command="remoteci-list-user")

    # purge commands
    p = subparsers.add_parser(
        "purge", help="Purge soft-deleted resources.", parents=[base_parser]
    )
    p.add_argument(
        "--resource", default=None, help="Comma separated list of resource to purge."
    )
    p.add_argument(
        "--force", default=False, action="store_true", help="Purge resources."
    )
    p.set_defaults(command="purge")

    args = parser.parse_args(args)
    return args



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.utils import active_string
from dciclient.v1.utils import validate_json

from dciclient.v1.api import component
from dciclient.v1.api import topic


def list(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset", "where"]}
    return topic.list_components(context, **params)


def create(context, args):
    params = {
        k: getattr(args, k)
        for k in [
            "name",
            "type",
            "canonical_project_name",
            "title",
            "message",
            "url",
            "topic_id",
            "state",
            "data",
        ]
    }
    params["data"] = validate_json(context, "data", params["data"])
    params["state"] = active_string(params["state"])
    return component.create(context, **params)


def delete(context, args):
    return component.delete(context, args.id)


def show(context, args):
    return component.get(context, args.id)


def file_upload(context, args):
    return component.file_upload(context, id=args.id, file_path=args.path)


def file_show(context, args):
    return component.file_get(context, id=args.id, file_id=args.file_id)


def file_download(context, args):
    params = {k: getattr(args, k) for k in ["id", "file_id", "target"]}
    return component.file_download(context, **params)


def file_list(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset", "where"]}
    return component.file_list(context, **params)


def file_delete(context, args):
    component.file_delete(context, id=args.id, file_id=args.file_id)


def update(context, args):
    component_info = component.get(context, args.id)

    etag = component_info.json()["component"]["etag"]

    return component.update(
        context, id=args.id, etag=etag, state=active_string(args.state)
    )


def attach_issue(context, args):
    return component.attach_issue(context, args.id, args.url)


def unattach_issue(context, args):
    return component.unattach_issue(context, args.id, args.issue_id)


def list_issues(context, args):
    params = {k: getattr(args, k) for k in ["id", "sort", "limit", "offset"]}
    return component.list_issues(context, **params)



# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json

import os
import os.path
from requests import compat

try:
    from urlparse import parse_qsl
    from urlparse import urlparse
except ImportError:
    from urllib.parse import parse_qsl
    from urllib.parse import urlparse
import requests
from requests.adapters import HTTPAdapter
from requests.auth import AuthBase
from requests.packages.urllib3.util.retry import Retry

from dciauth.request import AuthRequest
from dciauth.signature import Signature
from dciclient import version


class DciContextBase(object):
    API_VERSION = "api/v1"

    def __init__(self, dci_cs_url, max_retries=0, user_agent=None):
        self.session = self._build_http_session(user_agent, max_retries)
        self.dci_cs_api = "%s/%s" % (dci_cs_url, DciContext.API_VERSION)
        self.last_job_id = None

    @staticmethod
    def _build_http_session(user_agent, max_retries):
        session = requests.Session()
        session.headers.setdefault("Content-Type", "application/json")
        if not user_agent:
            user_agent = "python-dciclient_%s" % version.__version__
        session.headers["User-Agent"] = user_agent
        session.headers["Client-Version"] = "python-dciclient_%s" % version.__version__
        retries = Retry(total=max_retries, backoff_factor=0.1)
        session.mount("http://", HTTPAdapter(max_retries=retries))
        session.mount("https://", HTTPAdapter(max_retries=retries))

        return session


class DciContext(DciContextBase):
    def __init__(self, dci_cs_url, login, password, max_retries=0, user_agent=None):
        super(DciContext, self).__init__(
            dci_cs_url.rstrip("/"), max_retries, user_agent
        )
        self.login = login
        self.session.auth = (login, password)


def build_dci_context(
    dci_cs_url=None, dci_login=None, dci_password=None, user_agent=None, max_retries=80
):
    dci_cs_url = dci_cs_url or os.environ.get("DCI_CS_URL", "")
    dci_login = dci_login or os.environ.get("DCI_LOGIN", "")
    dci_password = dci_password or os.environ.get("DCI_PASSWORD", "")

    if not dci_cs_url or not dci_login or not dci_password:
        msg = (
            "Environment variables required: DCI_CS_URL=%s, "
            "DCI_LOGIN=%s, DCI_PASSWORD=%s" % (dci_cs_url, dci_login, dci_password)
        )
        raise Exception(msg)

    return DciContext(
        dci_cs_url.rstrip("/"),
        dci_login,
        dci_password,
        user_agent=user_agent,
        max_retries=max_retries,
    )


class DciSignatureAuth(AuthBase):
    """Signs the request for DCI API with signature authentication"""

    def __init__(self, client_id, api_secret):
        self.client_type, self.client_id = self.get_client_info(client_id)
        self.api_secret = api_secret

    @staticmethod
    def get_client_info(client_id):
        if client_id.find("/") == -1:
            return ["remoteci", client_id]
        return client_id.split("/")[:2]

    def __call__(self, r):
        url = urlparse(r.url)
        params = dict(parse_qsl(url.query))
        payload = self.get_payload(r)
        request = AuthRequest(
            method=r.method,
            endpoint=url.path,
            headers=r.headers,
            params=params,
            payload=payload,
        )
        headers = Signature(request).generate_headers(
            client_type=self.client_type,
            client_id=self.client_id,
            secret=self.api_secret,
        )
        r.headers.update(headers)
        return r

    def get_payload(self, r):
        try:
            if isinstance(r.body, compat.bytes):
                body = r.body.decode("utf-8")
            else:
                body = r.body
            return dict(json.loads(body or "{}"))
        except TypeError:
            return {}


class DciSignatureContext(DciContextBase):
    def __init__(
        self, dci_cs_url, client_id, api_secret, max_retries=0, user_agent=None
    ):
        super(DciSignatureContext, self).__init__(
            dci_cs_url.rstrip("/"), max_retries, user_agent
        )
        self.session.auth = DciSignatureAuth(client_id, api_secret)


def build_signature_context(
    dci_cs_url=None,
    dci_client_id=None,
    dci_api_secret=None,
    user_agent=None,
    max_retries=80,
):
    dci_cs_url = dci_cs_url or os.environ.get("DCI_CS_URL", "")
    dci_client_id = dci_client_id or os.environ.get("DCI_CLIENT_ID", "")
    dci_api_secret = dci_api_secret or os.environ.get("DCI_API_SECRET", "")

    if not dci_cs_url or not dci_client_id or not dci_api_secret:
        msg = (
            "Environment variables required: DCI_CS_URL, "
            "DCI_CLIENT_ID, DCI_API_SECRET"
        )
        raise Exception(msg)
    return DciSignatureContext(
        dci_cs_url,
        dci_client_id,
        dci_api_secret,
        user_agent=user_agent,
        max_retries=max_retries,
    )


class SsoContext(DciContextBase):
    def __init__(self, dci_cs_url, token, max_retries=0, user_agent=None):
        super(SsoContext, self).__init__(
            dci_cs_url.rstrip("/"), max_retries, user_agent
        )
        self.session.headers["Authorization"] = "Bearer %s" % token


def build_sso_context(
    dci_cs_url,
    sso_url,
    username,
    password,
    token,
    max_retries=0,
    user_agent=None,
    refresh=False,
):
    def _get_token_from_file(token_path):
        if os.path.exists(token_path):
            with open(token_path, "r") as f:
                return f.read()
        return None

    def _write_token_to_file(token_path, token):
        cache_folder = os.path.dirname(token_path)
        if not os.path.exists(cache_folder):
            os.makedirs(cache_folder)

        with open(token_path, "w") as f:
            f.write(token)

    def _get_token_from_server(sso_url, username, password):
        url = "%s/auth/realms/redhat-external/protocol/openid-connect/token" % sso_url
        data = {
            "client_id": "dci",
            "grant_type": "password",
            "username": username,
            "password": password,
        }
        result = requests.Session().post(url, data=data)
        return result.json()["access_token"]

    token_path = os.path.join(os.environ["HOME"], ".cache", "dci_token")
    token = token or _get_token_from_file(token_path)

    if not token or refresh:
        sso_url = sso_url or os.environ.get("SSO_URL", "").rstrip("/")
        username = username or os.environ.get("SSO_USERNAME", "")
        password = password or os.environ.get("SSO_PASSWORD", "")
        if not sso_url or not username or not password:
            msg = (
                "Environment variables required to build token: SSO_URL, "
                "SSO_USERNAME, SSO_PASSWORD or use SSO_TOKEN."
            )
            raise Exception(msg)
        token = _get_token_from_server(sso_url, username, password)
        _write_token_to_file(token_path, token)

    dci_cs_url = dci_cs_url or os.environ.get("DCI_CS_URL", "")
    return SsoContext(dci_cs_url, token, max_retries, user_agent)
